# coding=utf-8
# vim: set fileencoding=utf-8
# Mountpoint selector accordion and page classes
#
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Chris Lumens <clumens@redhat.com>
#


from pyanaconda.i18n import _
from pyanaconda.product import productName, productVersion
from pyanaconda.ui.gui.utils import escape_markup, really_hide, really_show
from pyanaconda.constants import DEFAULT_AUTOPART_TYPE
from pyanaconda.storage_utils import AUTOPART_CHOICES
from pyanaconda.ui.lib.disks import getDisks

from gi.repository.AnacondaWidgets import MountpointSelector
from gi.repository import Gtk
from blivet.size import Size

__all__ = ["DATA_DEVICE", "SYSTEM_DEVICE",
           "newSelectorFromDevice", "updateSelectorFromDevice",
           "Accordion",
           "Page", "UnknownPage", "CreateNewPage","CreateAutopartPage"]

DATA_DEVICE = 0
SYSTEM_DEVICE = 1
def updateSelectorFromDevice(selector, device, mountpoint=""):
    """Create a MountpointSelector from a Device object template.  This
       method should be used whenever constructing a new selector, or when
       setting a bunch of attributes on an existing selector.  For just
       changing the name or size, it's probably fine to do it by hand.

       This method returns the selector created.

       If given a selector parameter, attributes will be set on that object
       instead of creating a new one.  The optional mountpoint parameter
       allows for specifying the mountpoint if it cannot be determined from
       the device (like for a Root specifying an existing installation).
    """
    if hasattr(device.format, "mountpoint") and device.format.mountpoint is not None:
        mp = device.format.mountpoint
    elif mountpoint:
        mp = mountpoint
    elif device.format.name:
        mp = device.format.name
    else:
        mp = _("Unknown")

    selector.props.name = device.name
    selector.props.size = str(device.size)
    selector.props.mountpoint = mp
    selector.device = device

def newSelectorFromDevice(device, mountpoint=""):
    selector = MountpointSelector(device.name, str(device.size))
    selector._root = None
    updateSelectorFromDevice(selector, device, mountpoint)

    return selector

# An Accordion is a box that goes on the left side of the custom partitioning spoke.  It
# stores multiple expanders which are here called Pages.  These Pages correspond to
# individual installed OSes on the system plus some special ones.  When one Page is
# expanded, all others are collapsed.
class Accordion(Gtk.Box):
    def __init__(self):
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=12)
        self._expanders = []

    def addPage(self, contents, cb):
        label = Gtk.Label(label="""<span size='large' weight='bold' fgcolor='white'>%s</span>""" %
                          escape_markup(contents.pageTitle), use_markup=True,
                          xalign=0, yalign=0.5, wrap=True)

        expander = Gtk.Expander()
        expander.set_label_widget(label)
        expander.add(contents)

        self.add(expander)
        self._expanders.append(expander)
        expander.connect("activate", self._onExpanded, cb)
        expander.show_all()

    def _find_by_title(self, title):
        for e in self._expanders:
            if e.get_child().pageTitle == title:
                return e

        return None

    @property
    def allPages(self):
        return [e.get_child() for e in self._expanders]

    @property
    def allSelectors(self):
        return [s for p in self.allPages for s in p.members]

    @property
    def allMembers(self):
        for page in self.allPages:
            for member in page.members:
                yield (page, member)

    def expandPage(self, pageTitle):
        page = self._find_by_title(pageTitle)
        if not page:
            raise LookupError()

        if not page.get_expanded():
            page.emit("activate")

    def removePage(self, pageTitle):
        # First, remove the expander from the list of expanders we maintain.
        target = self._find_by_title(pageTitle)
        if not target:
            return

        self._expanders.remove(target)

        # Then, remove it from the box.
        self.remove(target)

    def removeAllPages(self):
        for e in self._expanders:
            self.remove(e)

        self._expanders = []

    def _onExpanded(self, obj, cb=None):
        if cb:
            cb(obj.get_child())

# A Page is a box that is stored in an Accordion.  It breaks down all the filesystems that
# comprise a single installed OS into two categories - Data filesystems and System filesystems.
# Each filesystem is described by a single MountpointSelector.
class Page(Gtk.Box):
    def __init__(self, title):
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=6)

        # Create the Data label and a box to store all its members in.
        self._dataBox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self._dataLabel = self._make_category_label(_("DATA"))
        really_hide(self._dataLabel)
        self._dataBox.add(self._dataLabel)
        self._dataBox.connect("add", self._onDataAdded)
        self._dataBox.connect("remove", self._onDataRemoved)
        self.add(self._dataBox)

        # Create the System label and a box to store all its members in.
        self._systemBox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self._systemBox.add(self._make_category_label(_("SYSTEM")))
        self.add(self._systemBox)

        self.members = []
        self.pageTitle = title

    def _make_category_label(self, name):
        label = Gtk.Label()
        label.set_markup("""<span fgcolor='dark grey' size='large' weight='bold'>%s</span>""" %
                escape_markup(name))
        label.set_halign(Gtk.Align.START)
        label.set_margin_left(24)
        return label

    def addSelector(self, device, cb, mountpoint=""):
        selector = newSelectorFromDevice(device, mountpoint=mountpoint)
        selector.connect("button-press-event", self._onSelectorClicked, cb)
        selector.connect("key-release-event", self._onSelectorClicked, cb)
        selector.connect("focus-in-event", self._onSelectorFocusIn, cb)
        selector.set_margin_bottom(6)
        self.members.append(selector)

        # pylint: disable=no-member
        if self._mountpointType(selector.props.mountpoint) == DATA_DEVICE:
            self._dataBox.add(selector)
        else:
            self._systemBox.add(selector)

        return selector

    def removeSelector(self, selector):
        if self._mountpointType(selector.props.mountpoint) == DATA_DEVICE:
            self._dataBox.remove(selector)
        else:
            self._systemBox.remove(selector)

        self.members.remove(selector)

    def _mountpointType(self, mountpoint):
        if not mountpoint or mountpoint in ["/", "/boot", "/boot/efi", "/tmp", "/usr", "/var",
                                            "biosboot", "prepboot", "swap"]:
            return SYSTEM_DEVICE
        else:
            return DATA_DEVICE

    def _onSelectorClicked(self, selector, event, cb):
        from gi.repository import Gdk

        if event and not event.type in [Gdk.EventType.BUTTON_PRESS, Gdk.EventType.KEY_RELEASE, Gdk.EventType.FOCUS_CHANGE]:
            return

        if event and event.type == Gdk.EventType.KEY_RELEASE and \
           event.keyval not in [Gdk.KEY_space, Gdk.KEY_Return, Gdk.KEY_ISO_Enter, Gdk.KEY_KP_Enter, Gdk.KEY_KP_Space]:
            return

        # Then, this callback will set up the right hand side of the screen to
        # show the details for the newly selected object.
        cb(selector)

    def _onSelectorFocusIn(self, selector, event, cb):
        # could be simple lambda, but this way it looks more similar to the
        # _onSelectorClicked
        cb(selector)

    def _onDataAdded(self, container, widget):
        really_show(self._dataLabel)

    def _onDataRemoved(self, container, widget):
        # This runs before widget is removed from container, so if it's the last
        # item then the container will still not be empty.
        if len(container.get_children()) == 1:
            really_hide(self._dataLabel)

class UnknownPage(Page):
    def __init__(self, title):
        # For this type of page, there's only one place to store members.
        # pylint: disable=super-init-not-called,non-parent-init-called
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=6)
        self.members = []
        self.pageTitle = title

    def addSelector(self, device, cb, mountpoint=""):
        selector = newSelectorFromDevice(device, mountpoint=mountpoint)
        selector.connect("button-press-event", self._onSelectorClicked, cb)
        selector.connect("key-release-event", self._onSelectorClicked, cb)

        self.members.append(selector)
        self.add(selector)

        return selector

    def removeSelector(self, selector):
        self.remove(selector)
        self.members.remove(selector)

# This is a special Page that is displayed when no new installation has been automatically
# created, and shows the user how to go about doing that.  The intention is that an instance
# of this class will be packed into the Accordion first and then when the new installation
# is created, it will be removed and replaced with a Page for it.
class CreateNewPage(Page):
    def __init__(self, title, createClickedCB, autopartTypeChangedCB, partitionsToReuse=True):
        # pylint: disable=super-init-not-called,non-parent-init-called
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=6)
        self.members = []
        self.pageTitle = title

        # Create a box where we store the "Here's how you create a new blah" info.
        self._createBox = Gtk.Grid()
        self._createBox.set_row_spacing(6)
        self._createBox.set_column_spacing(6)
        self._createBox.set_margin_left(16)

        label = Gtk.Label(label=_("You haven't created any mount points for your "
                            "%(product)s %(version)s installation yet.  "
                            "You can:") % {"product" : _(productName), "version" : productVersion},
                            wrap=True, xalign=0, yalign=0.5)
        #self._createBox.attach(label, 0, 0, 2, 1)

	# add by duanqi
	'''
        dot = Gtk.Label(label="•", xalign=0.5, yalign=0.4, hexpand=False)
        self._createBox.attach(dot, 0, 1, 1, 1)
	'''

        self._createNewButton = Gtk.Button(label=_("_Click here to create them automatically."))
        label = self._createNewButton.get_children()[0]
	Gtk.StyleContext.add_class(Gtk.Widget.get_style_context(self._createNewButton) ,"suggested-action")
	#label.set_markup(_("_Click here <b>to create them automatically</b>"))
	label.set_markup(_("<b>剩余空间自动创建分区</b>"))
        label.set_alignment(0, 0.5)
        label.set_hexpand(True)
        label.set_line_wrap(True)
        label.set_use_underline(True)
        # Create this now to pass into the callback.  It will be populated later
        # on in this method.
        store = Gtk.ListStore(str, int)
        combo = Gtk.ComboBox(model=store)
        cellrendr = Gtk.CellRendererText()
        combo.pack_start(cellrendr, True)
        combo.add_attribute(cellrendr, "text", 0)
        combo.connect("changed", autopartTypeChangedCB)

        self._createNewButton.set_has_tooltip(False)
        self._createNewButton.set_halign(Gtk.Align.START)
        self._createNewButton.connect("clicked", createClickedCB, combo)
        ##self._createNewButton.connect("activate-link", lambda *args: Gtk.true())
        self._createBox.attach(self._createNewButton, 0, 1, 2, 1)
        dot = Gtk.Label(label="•", xalign=0.5, yalign=0, hexpand=False)
        self._createBox.attach(dot, 0, 2, 1, 1)

        label = Gtk.Label(label=_("Create new mount points by clicking the '+' button."),
                          xalign=0, yalign=0.5, hexpand=True, wrap=True)
        self._createBox.attach(label, 1, 2, 1, 1)
	label.set_markup(_("Create new mount points by clicking the '+/-' button."))
        if partitionsToReuse:
            dot = Gtk.Label(label="•", xalign=0.5, yalign=0, hexpand=False)
            self._createBox.attach(dot, 0, 3, 1, 1)

            label = Gtk.Label(label=_("Or, assign new mount points to existing "
                                      "partitions after selecting them below."),
                              xalign=0, yalign=0.5, hexpand=True, wrap=True)
            self._createBox.attach(label, 1, 3, 1, 1)

        label = Gtk.Label(label=_("_New mount points will use the following partitioning scheme:"),
                          xalign=0, yalign=0.5, wrap=True, use_underline=True)
        ##self._createBox.attach(label, 0, 4, 2, 1)
        label.set_mnemonic_widget(combo)

        for item in (AUTOPART_CHOICES):
            itr = store.append([_(item[0]), item[1]])
            if item[1] == DEFAULT_AUTOPART_TYPE:
                default = itr

        combo.set_margin_left(18)
        combo.set_margin_right(18)
        combo.set_hexpand(False)
        combo.set_active_iter(default)
        ##self._createBox.attach(combo, 0, 5, 2, 1)

        self.add(self._createBox)
class CreateAutopartPage(Page):
    #def __init__(self,title, createClickedCB, autopartTypeChangedCB, partitionsToReuse=True):
    def __init__(self,selected_disks,size):
        # pylint: disable=super-init-not-called,non-parent-init-called
        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=6)
#        self.members = []
	self.selected_disks = selected_disks
	self.size_GB = int(size)
	self.OVER_SIZE = 0
        # Create a box where we store the "Here's how you create a new blah" info.
	self.vbox = Gtk.VBox(False,8)
	sw = Gtk.ScrolledWindow()
	#sw.set_shadow_type(Gtk.SHADOW_ETCHED_IN)
	#sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
	#self.vbox.pack_start(sw, True, True, 0)


	store = self.create_model()

        treeView = Gtk.TreeView(store)
	treeView.connect("row-activated", self.on_activated)
	#treeView.set_rules_hint(True)
	treeView.set_rules_hint(False)
	treeView.expand_all()
	treeView.set_show_expanders(True)
	treeView.size_request()
	treeView.set_show_expanders(True)
	#treeView.set_size_request(100,50)
	#treeView.set_expander_column()
	#treeView.set_fixed_height_mode(True)
        #sw.add(treeView)
	self.vbox.pack_start(treeView, True, True, 0)
	self.create_columns(treeView)
	#self.statusbar = Gtk.Statusbar()
	treeView.expand_all()

	#self.vbox.pack_start(self.statusbar, False, False, 0)
	self.add(self.vbox)

    def create_model(self):
	#actresses = self.get_autopart_device()
        import os

	self.disks = []
	#self.disks = getDisks(self.storage.devicetree)
	disk_name = "sda"
	if self.selected_disks:
	    disk_name = self.selected_disks
	else:
	    print "self.selected_disks[0] is none;can not set the autopart layout"
	    #return



	actresses = self.get_partition_layout()
	store = Gtk.ListStore(str, str, str,str)
	num = 1
	used_size = 0
	all_used_size = 0
	pre_size = "0"
	for act in actresses:
	    if act[2].endswith("GiB"):
		pre_size = act[2]
		used_size = used_size + int(act[2].strip("GiB"))
	        all_used_size =  all_used_size + int(act[2].strip("GiB"))
	    elif act[2].endswith("MiB"):
		pre_size = act[2]
		used_size = used_size + int(act[2].strip("MiB")) / 1024
	        all_used_size =  all_used_size + int(act[2].strip("MiB")) / 1024
	    elif act[2].startswith("0."):
		pre_size = str((self.size_GB - used_size - self.OVER_SIZE) * float(act[2])) + "GiB"
	        all_used_size =  all_used_size + (self.size_GB - used_size - self.OVER_SIZE) * float(act[2])
	    else:
		#act[2] = 1, all space
		pre_size = str(self.size_GB - all_used_size - self.OVER_SIZE) + "GiB"
	    if act[1] == "NONE":
		act[1] = "ext4"
	    store.append([disk_name + str(num), pre_size, act[1],"全盘安装"])
	    if num == 3 and not os.path.exists("/sys/firmware/efi"):
		num = num + 1
	    num = num + 1
	return store

    def create_columns(self, treeView):
	import gtk
	rendererText = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn("设备", rendererText, text=0)
	column.set_sort_column_id(0)
	column.set_expand(True)
	column.set_resizable(True)
	#column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
	treeView.append_column(column)
	treeView.set_expander_column(column)
	rendererText = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn("大小", rendererText, text=1)
	column.set_sort_column_id(1)
	column.set_expand(True)
	column.set_resizable(True)
	#column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
	treeView.append_column(column)
	treeView.set_expander_column(column)
	rendererText = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn("文件系统", rendererText, text=2)
	column.set_sort_column_id(2)
	column.set_expand(True)
	column.set_resizable(True)
	#column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
	treeView.append_column(column)
	treeView.set_expander_column(column)
	rendererText = Gtk.CellRendererText()
	column = Gtk.TreeViewColumn("安装方式", rendererText, text=3)
	column.set_sort_column_id(3)
	column.set_expand(True)
	column.set_resizable(True)
	#column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
	treeView.append_column(column)
	treeView.set_expander_column(column)
    def on_activated(self, widget, row, col):
	model = widget.get_model()
	text = model[row][0] + ", " + model[row][1] + ", " + model[row][2]
	#self.statusbar.push(0, text)

    def addAutopartPagePage(self, contents, cb):
        expander = Gtk.Expander()
        #expander.set_label_widget(label)
        expander.add(contents)

        self.add(expander)
        self._expanders.append(expander)
        #expander.connect("activate", self._onExpanded, cb)
        expander.show_all()

    def get_autopart_device(self):
	'''
	aar_all = []
	aar_simple = []
	for i in ("/dev/sda1","/dev/sda1","/dev/sda3","/dev/sda4","/dev/sda5"):
	    aar_simple.append("11")
	    aar_simple.append("30G")
	    aar_simple.append("all")
	    aar_simple.append("ext3")
	    aar_all.append(aar_simple)
	return aar_all
	'''
	print "------------------------"
	print self.selected_disks 
	arr_name = ["/dev/sda1","/dev/sda1","/dev/sda3","/dev/sda4","/dev/sda5"]
	arr_size = [0 for x in range(5)]
	arr_type = ["autopart" for x in range(5)]
	arr_filesystem = ["ext3" for x in range(5)]
	arr_all = [[0 for x in range(4)]for y in range(5)]
	if self.size_GB > 64:
	    arr_size[0] = 30 
	    arr_size[1] = 4
	    arr_size[2] = 30
	    arr_size[3] = self.size_GB - (arr_size[0] + arr_size[1] + arr_size[2])
	    arr_size[4] = self.size_GB - (arr_size[0] + arr_size[1] + arr_size[2]) 
	

	count = 0
	for a in range(5):
	    arr_all[a][0] = arr_name[a]
	for a in range(5):
	    arr_all[a][1] = str(arr_size[a]) + (" GiB")
	for a in range(5):
	    arr_all[a][2] = arr_type[a]
	for a in range(5):
	    arr_all[a][3] = arr_filesystem[a]
	    #count = count + 1

	return arr_all
        

    def get_partition_layout(self):
	import os
	file = ""
	partition_layout = []
	if os.uname()[4].startswith("x86") or os.uname()[4].startswith("i686"):
	    PAR_LARGE = "/etc/partition-240.txt"
            PAR_SMALL = "/etc/partition-120.txt"
	    if os.path.exists("/sys/firmware/efi"):
	        partition_layout.insert(0,['efi', 'efi', '200MiB'])
	        partition_layout.insert(1,['boot', 'ext4', '500MiB'])

        else:
            PAR_LARGE = "/run/install/repo/tools/partition-240.txt"
            PAR_SMALL = "/run/install/repo/tools/partition-120.txt"

	if self.size_GB > 223: #240G
	    file = PAR_LARGE
	    if not os.path.exists(file):
		return

	elif self.size_GB > 110: #120G
	    file = PAR_SMALL
	    if not os.path.exists(file):
		return
	else:
	    if self.size_GB > 93:
		if os.uname()[4].startswith("mips") and os.path.exists("/tmp/.overlay"):

		    ONLY_ROOT = ["root","ext4","30GiB"]
                    SWAP = ["swap","swap","4GiB"]
		    BACKUP = ["/srv","ext4","30GiB"]
		    HOME = ["home","ext4","1"]
                    partition_layout.append(ONLY_ROOT)
                    partition_layout.append(SWAP)
                    partition_layout.append(BACKUP)
                    partition_layout.append(HOME)
                    OVER = ["overlay","ext4","20GiB"]
                    partition_layout.append(OVER)
                    self.OVER_SIZE = 20
                    return partition_layout

		else:

		    ONLY_ROOT = ["root","ext4","30GiB"]
                    SWAP = ["swap","swap","4GiB"]
		    BACKUP = ["/srv","ext4","30GiB"]
		    HOME = ["home","ext4","1"]
                    partition_layout.append(ONLY_ROOT)
                    partition_layout.append(SWAP)
                    partition_layout.append(BACKUP)
                    partition_layout.append(HOME)
                    return partition_layout

	    elif self.size_GB > 50: #80G
		if os.uname()[4].startswith("mips") and os.path.exists("/tmp/.overlay"):
		
                    ONLY_ROOT = ["root","ext4","20GiB"]
                    SWAP = ["swap","swap","4GiB"]
                    WORK = ["/WORK","ext4","4GiB"]
                    DATA = ["/DATA","ext4","1"]
                    OVER = ["overlay","ext4","20GiB"]
                    self.OVER_SIZE = 20
                    partition_layout.append(ONLY_ROOT)
                    partition_layout.append(SWAP)
                    partition_layout.append(WORK)
                    partition_layout.append(DATA)
                    partition_layout.append(OVER)
                    return partition_layout
		else:
                    ONLY_ROOT = ["root","NONE","20GiB"]
                    SWAP = ["swap","swap","4GiB"]
                    WORK = ["/WORK","ext4","4GiB"]
                    DATA = ["/DATA","ext4","1"]
                    partition_layout.append(ONLY_ROOT)
                    partition_layout.append(SWAP)
                    partition_layout.append(WORK)
                    partition_layout.append(DATA)
                    return partition_layout
            elif self.size_GB >= 14:
                ONLY_ROOT = ["root","ext4","1"]
                partition_layout.append(ONLY_ROOT)
                return partition_layout
            else:
                return partition_layout

        file_txt=open(file)
        for line in file_txt:
            if line.strip() == "PRE_PARTITION_END":
                break
            partition_layout.append(line[14:].strip().split("_"))
	if self.size_GB > 223:
	    if os.uname()[4].startswith("mips") and os.path.exists("/tmp/.overlay") :
	    	partition_layout.append(["overlay","ext4","30GiB"])
                self.OVER_SIZE = 30
	elif self.size_GB > 110:
	    if os.uname()[4].startswith("mips") and os.path.exists("/tmp/.overlay") :
		partition_layout.append(["overlay","ext4","20GiB"])
                self.OVER_SIZE = 20

        #print partition_layout
	return partition_layout

